--[[
************************************************************************
Project				: Broker_AllStats
Author				: zhinjio
Project Revision	: r3
Project Date		: 20081212110236

File				: Broker_AllStats.lua
Commit Author		: zhinjio
Commit Revision		: 3
Commit Date			: 20081212110236
************************************************************************
Description	:
	Main addon file. 
TODO		:
************************************************************************
(see bottom of file for changelog)
************************************************************************
--]]

local MODNAME = "AllStats"
local FULLNAME = "Broker_AllStats"
local AllStats = LibStub( "AceAddon-3.0" ):NewAddon( MODNAME, "AceEvent-3.0" )
local L = LibStub:GetLibrary( "AceLocale-3.0" ):GetLocale( MODNAME )
local AceConfig 		= LibStub("AceConfig-3.0")
local AceConfigReg 		= LibStub("AceConfigRegistry-3.0")
local AceConfigDialog 	= LibStub("AceConfigDialog-3.0")

_G[ MODNAME ] = AllStats
local addon = AllStats

local addonversion = "r3"
local addonauthor = "zhinjio"
local builddate = "20081212110236"

local modularOptions = {}
local tooltip
local LTT = LibStub:GetLibrary( "LibQTip-1.0" )

-- addon globals
-- addon.someValue = 0
addon.me = ""

local db
local cachedValues = {}

function addon.Print( msg )
	local _, fh = DEFAULT_CHAT_FRAME:GetFont()
	fh = math.ceil( fh )
	local icon = "\124TInterface\\Addons\\" .. FULLNAME .. "\\icon:" .. fh .. "\124t"
	if ( msg ~= nil ) then
	   if( DEFAULT_CHAT_FRAME ) then
	      DEFAULT_CHAT_FRAME:AddMessage( icon .. " " .. MODNAME .. " : " .. msg, 0.6, 1.0, 1.0 )
	   end
	end
end

function addon.Debug( msg )
	if ( msg ~= nil ) then
		if ( db.options.display.debug ) then
			addon.Print( "DEBUG: " .. msg )
		end
	end
end

-- coloring agents
--local function Orange ( text )
--	return "|cffdea51c" .. text .. "|r"
--end

local function HideTooltip()
	tooltip:SetScript( "OnLeave", nil )
	tooltip:Hide()
end

local function drawTooltip()
	tooltip:Hide()
	tooltip:Clear()

	-- tooltip display code here...

	tooltip:AddLine(" ")
	tooltip:AddLine( L["Left-Click to Refresh Values"] )
	tooltip:AddLine( L["Right-click for Configuration"] )
end

-- LDB stuff
local LDB = LibStub( "LibDataBroker-1.1" )
local launcher = LDB:NewDataObject( MODNAME, {
	type = "data source",
	text = " ",
	label = FULLNAME,
	icon = "Interface\\Addons\\" .. FULLNAME .. "\\icon",
	OnClick = function( clickedframe, button )
				if button == "RightButton" then
					addon:ShowConfig()
				else
					drawTooltip()
					tooltip:SmartAnchorTo( clickedframe )
					tooltip:Show()
				end
	end,
	OnEnter = function ( self )
		drawTooltip()
		tooltip:EnableMouse()
		tooltip:SmartAnchorTo( self )
		tooltip:Show()
	end,
	OnLeave = function ( self )
		if MouseIsOver( tooltip ) then
			tooltip:SetScript( 'OnLeave', HideTooltip )
		else
			HideTooltip()
		end
	end,
} )

function addon.upgradeDB()
	local olddb = addon.db.factionrealm[addon.me]
	if ( olddb.options.display.dbVersion == "1.0" ) then
		olddb.options.display.text = true
		-- tooltip groups
		olddb.options.tooltipgroups = {}
		olddb.options.tooltipgroups.basic = true
		olddb.options.tooltipgroups.resist = true
		olddb.options.tooltipgroups.defense = true
		olddb.options.tooltipgroups.melee = true
		olddb.options.tooltipgroups.ranged = true
		olddb.options.tooltipgroups.spell = true
		-- text display
		olddb.options.tooltipgroups.shuffle = true
		olddb.options.tooltipgroups.interval = 10
		olddb.options.textdisplay = {}
			-- Basic group
			olddb.options.textdisplay.strength = false
			olddb.options.textdisplay.agility = false
			olddb.options.textdisplay.stamina = false
			olddb.options.textdisplay.intellect = false
			olddb.options.textdisplay.spirit = false
			olddb.options.textdisplay.health = false
			olddb.options.textdisplay.power = false
			-- Defense Group
			olddb.options.textdisplay.armor = false
			olddb.options.textdisplay.defense = false
			olddb.options.textdisplay.defensebonus = false
			olddb.options.textdisplay.defenserating = false
			olddb.options.textdisplay.parryrating = false
			olddb.options.textdisplay.parrybonus = false
			olddb.options.textdisplay.parrychance = false
			olddb.options.textdisplay.dodgerating = false
			olddb.options.textdisplay.dodgebonus = false
			olddb.options.textdisplay.dodgechance = false
			olddb.options.textdisplay.blockrating = false
			olddb.options.textdisplay.blockbonus = false
			olddb.options.textdisplay.blockpercent = false
			olddb.options.textdisplay.avoidance = false
			-- Resistance Group
			olddb.options.textdisplay.holyresist = false
			olddb.options.textdisplay.fireresist = false
			olddb.options.textdisplay.natureresist = false
			olddb.options.textdisplay.frostresist = false
			olddb.options.textdisplay.shadowresist = false
			olddb.options.textdisplay.arcaneresist = false
			-- Melee Group
			olddb.options.textdisplay.maindmg = false
			olddb.options.textdisplay.offdmg= false
			olddb.options.textdisplay.mainspeed = false
			olddb.options.textdisplay.offspeed = false
			olddb.options.textdisplay.attpower = false
			olddb.options.textdisplay.hitrating = false
			olddb.options.textdisplay.hitbonus = false
			olddb.options.textdisplay.critrating = false
			olddb.options.textdisplay.critbonus = false
			olddb.options.textdisplay.critchance = false
			olddb.options.textdisplay.hasterating = false
			olddb.options.textdisplay.hastebonus = false
			olddb.options.textdisplay.armorpen = false
			olddb.options.textdisplay.expertrating = false
			olddb.options.textdisplay.expertbonus = false
			olddb.options.textdisplay.beparrychance = false
			-- Ranged Group
			olddb.options.textdisplay.rangespeed = false
			olddb.options.textdisplay.rangedmg = false
			olddb.options.textdisplay.rangeAP = false
			olddb.options.textdisplay.rangehitrating = false
			olddb.options.textdisplay.rangehitbonus = false
			olddb.options.textdisplay.rangecritrating = false
			olddb.options.textdisplay.rangecritbonus = false
			olddb.options.textdisplay.rangecritchance = false
			olddb.options.textdisplay.rangehasterating = false
			olddb.options.textdisplay.rangehastebonus = false
			-- Spell Group
			olddb.options.textdisplay.holydmg = false
			olddb.options.textdisplay.firedmg = false
			olddb.options.textdisplay.naturedmg = false
			olddb.options.textdisplay.frostdmg = false
			olddb.options.textdisplay.shadowdmg = false
			olddb.options.textdisplay.arcanedmg = false
			olddb.options.textdisplay.healing = false
			olddb.options.textdisplay.spellhitrating = false
			olddb.options.textdisplay.spellhitbonus = false
			olddb.options.textdisplay.spellcritrating = false
			olddb.options.textdisplay.spellcritbonus = false
			olddb.options.textdisplay.holycrit = false
			olddb.options.textdisplay.firecrit = false
			olddb.options.textdisplay.naturecrit = false
			olddb.options.textdisplay.frostcrit = false
			olddb.options.textdisplay.shadowcrit = false
			olddb.options.textdisplay.arcanecrit = false
			olddb.options.textdisplay.spellhasterating = false
			olddb.options.textdisplay.spellhastebonus = false
			olddb.options.textdisplay.spellpenetration = false
			olddb.options.textdisplay.manaregen = false
			olddb.options.textdisplay.castingregen = false
	end
	if ( olddb.options.display.dbVersion == "1.1" ) then
		olddb.options.tooltipgroups.shuffle = nil
		olddb.options.tooltipgroups.interval = nil
		olddb.options.textdisplay.shuffle = true
		olddb.options.textdisplay.interval = 10
		olddb.options.display.dbVersion = "1.2"
		addon.Print( string.format( L["Database upgraded to %s"], "1.2" ) )
	end
end

-- options stuff
local displayOptions = nil
local tooltipOptions = nil
local textOptions = nil
local function giveDisplay()
	if (not displayOptions) then
		displayOptions = {
			order = 2,
			type = "group",
			name = L["Display Options"],
			desc = L["DISPLAY_OPTIONS_DESC"],
			args = {
				header1 = {
					order	= 21,
					type	= "header",
					name	= L["Display Options"],
				},
				desc = {
					order	= 22,
					type	= "description",
					name	= L["DISPLAY_OPTIONS_DESC"],
				},
				sound = {
					order	= 23,
					type	= "toggle",
					name	= L["Enable DEBUG"],
					desc	= L["DISPLAY_DEBUG_DESC"],
					get		= function() return db.options.display.debug end,
					set		= function() db.options.display.debug = not db.options.display.debug end,
				},
				tooltipscale = {
					order	= 24,
					type	= "range",
					name	= L["Tooltip Scale"],
					desc	= L["TOOLTIP_SCALE_DESC"],
					min		= .7,
					max		= 1.3,
					step	= .05,
					bigStep = .05,
					get		= function() return db.options.display.tipscale end,
					set		= function( info, v ) 
								db.options.display.tipscale = v
								tooltip:SetScale( v )
							  end,
				},
				sound = {
					order	= 25,
					type	= "toggle",
					name	= L["Display Text"],
					desc	= L["DISPLAY_TEXT_DESC"],
					get		= function() return db.options.display.text end,
					set		= function() db.options.display.text = not db.options.display.text end,
				},
			},
		}
	end
	return displayOptions
end

local function giveTooltip()
	if (not tooltipOptions) then
		tooltipOptions = {
			order = 3,
			type = "group",
			name = L["Tooltip Options"],
			desc = L["TOOLTIP_OPTIONS_DESC"],
			args = {
				header1 = {
					order	= 31,
					type	= "header",
					name	= L["Tooltip Options"],
				},
				desc = {
					order	= 32,
					type	= "description",
					name	= L["TOOLTIP_OPTIONS_DESC"],
				},
				basic = {
					order	= 33,
					type	= "toggle",
					name	= L["Basic Group"],
					desc	= L["TOOLTIP_BASIC_DESC"],
					get		= function() return db.options.tooltipgroups.basic end,
					set		= function() db.options.tooltipgroups.basic = not db.options.tooltipgroups.basic end,
				},
				resist = {
					order	= 34,
					type	= "toggle",
					name	= L["Resistance Group"],
					desc	= L["TOOLTIP_RESIST_DESC"],
					get		= function() return db.options.tooltipgroups.resist end,
					set		= function() db.options.tooltipgroups.resist = not db.options.tooltipgroups.resist end,
				},
				defense = {
					order	= 35,
					type	= "toggle",
					name	= L["Defense Group"],
					desc	= L["TOOLTIP_DEFENSE_DESC"],
					get		= function() return db.options.tooltipgroups.defense end,
					set		= function() db.options.tooltipgroups.defense = not db.options.tooltipgroups.defense end,
				},
				melee = {
					order	= 36,
					type	= "toggle",
					name	= L["Melee Group"],
					desc	= L["TOOLTIP_MELEE_DESC"],
					get		= function() return db.options.tooltipgroups.melee end,
					set		= function() db.options.tooltipgroups.melee = not db.options.tooltipgroups.melee end,
				},
				ranged = {
					order	= 37,
					type	= "toggle",
					name	= L["Ranged Group"],
					desc	= L["TOOLTIP_RANGED_DESC"],
					get		= function() return db.options.tooltipgroups.ranged end,
					set		= function() db.options.tooltipgroups.ranged = not db.options.tooltipgroups.ranged end,
				},
				spell = {
					order	= 38,
					type	= "toggle",
					name	= L["Spell Group"],
					desc	= L["TOOLTIP_SPELL_DESC"],
					get		= function() return db.options.tooltipgroups.spell end,
					set		= function() db.options.tooltipgroups.spell = not db.options.tooltipgroups.spell end,
				},
			},
		}
	end
	return tooltipOptions
end

local function giveText()
	if (not textOptions) then
		textOptions = {
			order = 4,
			type = "group",
			name = L["Text Options"],
			desc = L["TEXT_OPTIONS_DESC"],
			args = {
				header1 = {
					order	= 41,
					type	= "header",
					name	= L["Text Options"],
				},
				desc = {
					order	= 42,
					type	= "description",
					name	= L["TEXT_OPTIONS_DESC"],
				},
				shuffle = {
					order	= 43,
					type	= "toggle",
					name	= L["Shuffle Values"],
					desc	= L["TEXT_SHUFFLE_DESC"],
					get		= function() return db.options.textdisplay.shuffle end,
					set		= function() db.options.textdisplay.shuffle = not db.options.textdisplay.shuffle end,
					disabled = function() return not db.options.display.text end,
				},
				interval = {
					order	= 44,
					type	= "range",
					name	= L["Shuffle Interval"],
					desc	= L["TEXT_INTERVAL_DESC"],
					min		= 5,
					max		= 30,
					step	= 1,
					bigStep = 1,
					get		= function() return db.options.textdisplay.interval end,
					set		= function( info, v ) db.options.textdisplay.interval = v end,
					disabled = function() return not db.options.display.text end,
				},
				header2 = {
					order	= 51,
					type	= "header",
					name	= L["Basic Group"],
				},
				str = {
					order	= 52,
					type	= "toggle",
					name	= L["Strength"],
					desc	= L["TEXT_STR_DESC"],
					get		= function() return db.options.textdisplay.strength end,
					set		= function() db.options.textdisplay.strength = not db.options.textdisplay.strength end,
					disabled = function() return not db.options.display.text end,
				},
				agi = {
					order	= 53,
					type	= "toggle",
					name	= L["Agility"],
					desc	= L["TEXT_AGI_DESC"],
					get		= function() return db.options.textdisplay.agility end,
					set		= function() db.options.textdisplay.agility = not db.options.textdisplay.agility end,
					disabled = function() return not db.options.display.text end,
				},
				sta = {
					order	= 54,
					type	= "toggle",
					name	= L["Stamina"],
					desc	= L["TEXT_STA_DESC"],
					get		= function() return db.options.textdisplay.stamina end,
					set		= function() db.options.textdisplay.stamina = not db.options.textdisplay.stamina end,
					disabled = function() return not db.options.display.text end,
				},
				int = {
					order	= 55,
					type	= "toggle",
					name	= L["Intellect"],
					desc	= L["TEXT_INT_DESC"],
					get		= function() return db.options.textdisplay.intellect end,
					set		= function() db.options.textdisplay.intellect = not db.options.textdisplay.intellect end,
					disabled = function() return not db.options.display.text end,
				},
				spi = {
					order	= 56,
					type	= "toggle",
					name	= L["Spirit"],
					desc	= L["TEXT_SPI_DESC"],
					get		= function() return db.options.textdisplay.spirit end,
					set		= function() db.options.textdisplay.spirit = not db.options.textdisplay.spirit end,
					disabled = function() return not db.options.display.text end,
				},
				health = {
					order	= 57,
					type	= "toggle",
					name	= L["Health"],
					desc	= L["TEXT_HEALTH_DESC"],
					get		= function() return db.options.textdisplay.health end,
					set		= function() db.options.textdisplay.health = not db.options.textdisplay.health end,
					disabled = function() return not db.options.display.text end,
				},
				power = {
					order	= 58,
					type	= "toggle",
					name	= L["Power"],
					desc	= L["TEXT_POWER_DESC"],
					get		= function() return db.options.textdisplay.power end,
					set		= function() db.options.textdisplay.power = not db.options.textdisplay.power end,
					disabled = function() return not db.options.display.text end,
				},
				header3 = {
					order	= 61,
					type	= "header",
					name	= L["Defense Group"],
				},
				armor = {
					order	= 62,
					type	= "toggle",
					name	= L["Armor"],
					desc	= L["TEXT_ARMOR_DESC"],
					get		= function() return db.options.textdisplay.armor end,
					set		= function() db.options.textdisplay.armor = not db.options.textdisplay.armor end,
					disabled = function() return not db.options.display.text end,
				},
				defense = {
					order	= 63,
					type	= "toggle",
					name	= L["Defense"],
					desc	= L["TEXT_DEF_DESC"],
					get		= function() return db.options.textdisplay.defense end,
					set		= function() db.options.textdisplay.defense = not db.options.textdisplay.defense end,
					disabled = function() return not db.options.display.text end,
				},
				defensebonus = {
					order	= 64,
					type	= "toggle",
					name	= L["Defense Bonus"],
					desc	= L["TEXT_DEFBONUS_DESC"],
					get		= function() return db.options.textdisplay.defensebonus end,
					set		= function() db.options.textdisplay.defensebonus = not db.options.textdisplay.defensebonus end,
					disabled = function() return not db.options.display.text end,
				},
				defenserating = {
					order	= 65,
					type	= "toggle",
					name	= L["Defense Rating"],
					desc	= L["TEXT_DEFRATING_DESC"],
					get		= function() return db.options.textdisplay.defenserating end,
					set		= function() db.options.textdisplay.defenserating = not db.options.textdisplay.defenserating end,
					disabled = function() return not db.options.display.text end,
				},
				parryrating = {
					order	= 66,
					type	= "toggle",
					name	= L["Parry Rating"],
					desc	= L["TEXT_PARRYRATING_DESC"],
					get		= function() return db.options.textdisplay.parryrating end,
					set		= function() db.options.textdisplay.parryrating = not db.options.textdisplay.parryrating end,
					disabled = function() return not db.options.display.text end,
				},
				parrybonus = {
					order	= 67,
					type	= "toggle",
					name	= L["Parry Bonus"],
					desc	= L["TEXT_PARRYBONUS_DESC"],
					get		= function() return db.options.textdisplay.parrybonus end,
					set		= function() db.options.textdisplay.parrybonus = not db.options.textdisplay.parrybonus end,
					disabled = function() return not db.options.display.text end,
				},
				parrychance = {
					order	= 68,
					type	= "toggle",
					name	= L["Parry Chance"],
					desc	= L["TEXT_PARRYCHANCE_DESC"],
					get		= function() return db.options.textdisplay.parrychance end,
					set		= function() db.options.textdisplay.parrychance = not db.options.textdisplay.parrychance end,
					disabled = function() return not db.options.display.text end,
				},
				dodgerating = {
					order	= 69,
					type	= "toggle",
					name	= L["Dodge Rating"],
					desc	= L["TEXT_DODGERATING_DESC"],
					get		= function() return db.options.textdisplay.dodgerating end,
					set		= function() db.options.textdisplay.dodgerating = not db.options.textdisplay.dodgerating end,
					disabled = function() return not db.options.display.text end,
				},
				dodgebonus = {
					order	= 70,
					type	= "toggle",
					name	= L["Dodge Bonus"],
					desc	= L["TEXT_DODGEBONUS_DESC"],
					get		= function() return db.options.textdisplay.dodgebonus end,
					set		= function() db.options.textdisplay.dodgebonus = not db.options.textdisplay.dodgebonus end,
					disabled = function() return not db.options.display.text end,
				},
				dodgechance = {
					order	= 71,
					type	= "toggle",
					name	= L["Dodge Chance"],
					desc	= L["TEXT_DODGECHANCE_DESC"],
					get		= function() return db.options.textdisplay.dodgechance end,
					set		= function() db.options.textdisplay.dodgechance = not db.options.textdisplay.dodgechance end,
					disabled = function() return not db.options.display.text end,
				},
				blockrating = {
					order	= 72,
					type	= "toggle",
					name	= L["Block Rating"],
					desc	= L["TEXT_BLOCKRATING_DESC"],
					get		= function() return db.options.textdisplay.blockrating end,
					set		= function() db.options.textdisplay.blockrating = not db.options.textdisplay.blockrating end,
					disabled = function() return not db.options.display.text end,
				},
				blockbonus = {
					order	= 73,
					type	= "toggle",
					name	= L["Block Bonus"],
					desc	= L["TEXT_BLOCKBONUS_DESC"],
					get		= function() return db.options.textdisplay.blockbonus end,
					set		= function() db.options.textdisplay.blockbonus = not db.options.textdisplay.blockbonus end,
					disabled = function() return not db.options.display.text end,
				},
				blockpercent = {
					order	= 74,
					type	= "toggle",
					name	= L["Block Percentage"],
					desc	= L["TEXT_BLOCKPERCENT_DESC"],
					get		= function() return db.options.textdisplay.blockpercent end,
					set		= function() db.options.textdisplay.blockpercent = not db.options.textdisplay.blockpercent end,
					disabled = function() return not db.options.display.text end,
				},
				avoidance = {
					order	= 75,
					type	= "toggle",
					name	= L["Avoidance"],
					desc	= L["TEXT_AVOIDANCE_DESC"],
					get		= function() return db.options.textdisplay.avoidance end,
					set		= function() db.options.textdisplay.avoidance = not db.options.textdisplay.avoidance end,
					disabled = function() return not db.options.display.text end,
				},
				header4 = {
					order	= 81,
					type	= "header",
					name	= L["Resistance Group"],
				},
				holyresist = {
					order	= 82,
					type	= "toggle",
					name	= L["Holy Resistance"],
					desc	= L["TEXT_HOLYRESIST_DESC"],
					get		= function() return db.options.textdisplay.holyresist end,
					set		= function() db.options.textdisplay.holyresist = not db.options.textdisplay.holyresist end,
					disabled = function() return not db.options.display.text end,
				},
				fireresist = {
					order	= 83,
					type	= "toggle",
					name	= L["Fire Resistance"],
					desc	= L["TEXT_FIRERESIST_DESC"],
					get		= function() return db.options.textdisplay.fireresist end,
					set		= function() db.options.textdisplay.fireresist = not db.options.textdisplay.fireresist end,
					disabled = function() return not db.options.display.text end,
				},
				natureresist = {
					order	= 84,
					type	= "toggle",
					name	= L["Nature Resistance"],
					desc	= L["TEXT_NATURERESIST_DESC"],
					get		= function() return db.options.textdisplay.natureresist end,
					set		= function() db.options.textdisplay.natureresist = not db.options.textdisplay.natureresist end,
					disabled = function() return not db.options.display.text end,
				},
				frostresist = {
					order	= 85,
					type	= "toggle",
					name	= L["Frost Resistance"],
					desc	= L["TEXT_FROSTRESIST_DESC"],
					get		= function() return db.options.textdisplay.frostresist end,
					set		= function() db.options.textdisplay.frostresist = not db.options.textdisplay.frostresist end,
					disabled = function() return not db.options.display.text end,
				},
				shadowresist = {
					order	= 86,
					type	= "toggle",
					name	= L["Shadow Resistance"],
					desc	= L["TEXT_SHADOWRESIST_DESC"],
					get		= function() return db.options.textdisplay.shadowresist end,
					set		= function() db.options.textdisplay.shadowresist = not db.options.textdisplay.shadowresist end,
					disabled = function() return not db.options.display.text end,
				},
				arcaneresist = {
					order	= 87,
					type	= "toggle",
					name	= L["Arcane Resistance"],
					desc	= L["TEXT_ARCANERESIST_DESC"],
					get		= function() return db.options.textdisplay.arcaneresist end,
					set		= function() db.options.textdisplay.arcaneresist = not db.options.textdisplay.arcaneresist end,
					disabled = function() return not db.options.display.text end,
				},
				header5 = {
					order	= 91,
					type	= "header",
					name	= L["Melee Group"],
				},
				maindmg = {
					order	= 92,
					type	= "toggle",
					name	= L["Main Hand Damage"],
					desc	= L["TEXT_MAINDMG_DESC"],
					get		= function() return db.options.textdisplay.maindmg end,
					set		= function() db.options.textdisplay.maindmg = not db.options.textdisplay.maindmg end,
					disabled = function() return not db.options.display.text end,
				},
				offdmg = {
					order	= 93,
					type	= "toggle",
					name	= L["Off Hand Damage"],
					desc	= L["TEXT_OFFDMG_DESC"],
					get		= function() return db.options.textdisplay.offdmg end,
					set		= function() db.options.textdisplay.offdmg = not db.options.textdisplay.offdmg end,
					disabled = function() return not db.options.display.text end,
				},
				mainspeed = {
					order	= 94,
					type	= "toggle",
					name	= L["Main Hand Speed"],
					desc	= L["TEXT_MAINSPEED_DESC"],
					get		= function() return db.options.textdisplay.mainspeed end,
					set		= function() db.options.textdisplay.mainspeed = not db.options.textdisplay.mainspeed end,
					disabled = function() return not db.options.display.text end,
				},
				offspeed = {
					order	= 95,
					type	= "toggle",
					name	= L["Off Hand Speed"],
					desc	= L["TEXT_OFFSPEED_DESC"],
					get		= function() return db.options.textdisplay.offspeed end,
					set		= function() db.options.textdisplay.offspeed = not db.options.textdisplay.offspeed end,
					disabled = function() return not db.options.display.text end,
				},
				attpower = {
					order	= 96,
					type	= "toggle",
					name	= L["Attack Power"],
					desc	= L["TEXT_ATTPOWER_DESC"],
					get		= function() return db.options.textdisplay.attpower end,
					set		= function() db.options.textdisplay.attpower = not db.options.textdisplay.attpower end,
					disabled = function() return not db.options.display.text end,
				},
				hitrating = {
					order	= 97,
					type	= "toggle",
					name	= L["Hit Rating"],
					desc	= L["TEXT_HITRATING_DESC"],
					get		= function() return db.options.textdisplay.hitrating end,
					set		= function() db.options.textdisplay.hitrating = not db.options.textdisplay.hitrating end,
					disabled = function() return not db.options.display.text end,
				},
				hitbonus = {
					order	= 98,
					type	= "toggle",
					name	= L["Hit Bonus"],
					desc	= L["TEXT_HITBONUS_DESC"],
					get		= function() return db.options.textdisplay.hitbonus end,
					set		= function() db.options.textdisplay.hitbonus = not db.options.textdisplay.hitbonus end,
					disabled = function() return not db.options.display.text end,
				},
				critrating = {
					order	= 99,
					type	= "toggle",
					name	= L["Crit Rating"],
					desc	= L["TEXT_CRITRATING_DESC"],
					get		= function() return db.options.textdisplay.critrating end,
					set		= function() db.options.textdisplay.critrating = not db.options.textdisplay.critrating end,
					disabled = function() return not db.options.display.text end,
				},
				critbonus = {
					order	= 100,
					type	= "toggle",
					name	= L["Crit Bonus"],
					desc	= L["TEXT_CRITBONUS_DESC"],
					get		= function() return db.options.textdisplay.critbonus end,
					set		= function() db.options.textdisplay.critbonus = not db.options.textdisplay.critbonus end,
					disabled = function() return not db.options.display.text end,
				},
				critchance = {
					order	= 101,
					type	= "toggle",
					name	= L["Crit Chance"],
					desc	= L["TEXT_CRITCHANCE_DESC"],
					get		= function() return db.options.textdisplay.critchance end,
					set		= function() db.options.textdisplay.critchance = not db.options.textdisplay.critchance end,
					disabled = function() return not db.options.display.text end,
				},
				hasterating = {
					order	= 102,
					type	= "toggle",
					name	= L["Haste Rating"],
					desc	= L["TEXT_HASTERATING_DESC"],
					get		= function() return db.options.textdisplay.hasterating end,
					set		= function() db.options.textdisplay.hasterating = not db.options.textdisplay.hasterating end,
					disabled = function() return not db.options.display.text end,
				},
				hastebonus = {
					order	= 103,
					type	= "toggle",
					name	= L["Haste Bonus"],
					desc	= L["TEXT_HASTEBONUS_DESC"],
					get		= function() return db.options.textdisplay.hastebonus end,
					set		= function() db.options.textdisplay.hastebonus = not db.options.textdisplay.hastebonus end,
					disabled = function() return not db.options.display.text end,
				},
				armorpen = {
					order	= 104,
					type	= "toggle",
					name	= L["Armor Penetration"],
					desc	= L["TEXT_ARMORPEN_DESC"],
					get		= function() return db.options.textdisplay.armorpen end,
					set		= function() db.options.textdisplay.armorpen = not db.options.textdisplay.armorpen end,
					disabled = function() return not db.options.display.text end,
				},
				expertrating = {
					order	= 105,
					type	= "toggle",
					name	= L["Expertise Rating"],
					desc	= L["TEXT_EXPERTRATING_DESC"],
					get		= function() return db.options.textdisplay.expertrating end,
					set		= function() db.options.textdisplay.expertrating = not db.options.textdisplay.expertrating end,
					disabled = function() return not db.options.display.text end,
				},
				expertbonus = {
					order	= 106,
					type	= "toggle",
					name	= L["Expertise Bonus"],
					desc	= L["TEXT_EXPERTBONUS_DESC"],
					get		= function() return db.options.textdisplay.expertbonus end,
					set		= function() db.options.textdisplay.expertbonus = not db.options.textdisplay.expertbonus end,
					disabled = function() return not db.options.display.text end,
				},
				beparrychance = {
					order	= 107,
					type	= "toggle",
					name	= L["Be Parried Chance"],
					desc	= L["TEXT_BEPARRYCHANCE_DESC"],
					get		= function() return db.options.textdisplay.beparrychance end,
					set		= function() db.options.textdisplay.beparrychance = not db.options.textdisplay.beparrychance end,
					disabled = function() return not db.options.display.text end,
				},
				header6 = {
					order	= 111,
					type	= "header",
					name	= L["Ranged Group"],
				},
				rangespeed = {
					order	= 112,
					type	= "toggle",
					name	= L["Ranged Speed"],
					desc	= L["TEXT_RANGESPEED_DESC"],
					get		= function() return db.options.textdisplay.rangespeed end,
					set		= function() db.options.textdisplay.rangespeed = not db.options.textdisplay.rangespeed end,
					disabled = function() return not db.options.display.text end,
				},
				rangedmg = {
					order	= 113,
					type	= "toggle",
					name	= L["Ranged Damage"],
					desc	= L["TEXT_RANGEDMG_DESC"],
					get		= function() return db.options.textdisplay.rangedmg end,
					set		= function() db.options.textdisplay.rangedmg = not db.options.textdisplay.rangedmg end,
					disabled = function() return not db.options.display.text end,
				},
				rangeAP = {
					order	= 114,
					type	= "toggle",
					name	= L["Ranged Attack Power"],
					desc	= L["TEXT_RANGEAP_DESC"],
					get		= function() return db.options.textdisplay.rangeAP end,
					set		= function() db.options.textdisplay.rangeAP = not db.options.textdisplay.rangeAP end,
					disabled = function() return not db.options.display.text end,
				},
				rangehitrating = {
					order	= 115,
					type	= "toggle",
					name	= L["Ranged Hit Rating"],
					desc	= L["TEXT_RANGEHITRATING_DESC"],
					get		= function() return db.options.textdisplay.rangehitrating end,
					set		= function() db.options.textdisplay.rangehitrating = not db.options.textdisplay.rangehitrating end,
					disabled = function() return not db.options.display.text end,
				},
				rangehitbonus = {
					order	= 116,
					type	= "toggle",
					name	= L["Ranged Hit Bonus"],
					desc	= L["TEXT_RANGEHITBONUS_DESC"],
					get		= function() return db.options.textdisplay.rangehitbonus end,
					set		= function() db.options.textdisplay.rangehitbonus = not db.options.textdisplay.rangehitbonus end,
					disabled = function() return not db.options.display.text end,
				},
				rangecritrating = {
					order	= 117,
					type	= "toggle",
					name	= L["Ranged Crit Rating"],
					desc	= L["TEXT_RANGECRITRATING_DESC"],
					get		= function() return db.options.textdisplay.rangecritrating end,
					set		= function() db.options.textdisplay.rangecritrating = not db.options.textdisplay.rangecritrating end,
					disabled = function() return not db.options.display.text end,
				},
				rangecritbonus = {
					order	= 118,
					type	= "toggle",
					name	= L["Ranged Crit Bonus"],
					desc	= L["TEXT_RANGECRITBONUS_DESC"],
					get		= function() return db.options.textdisplay.rangecritbonus end,
					set		= function() db.options.textdisplay.rangecritbonus = not db.options.textdisplay.rangecritbonus end,
					disabled = function() return not db.options.display.text end,
				},
				rangecritchance = {
					order	= 119,
					type	= "toggle",
					name	= L["Ranged Crit Chance"],
					desc	= L["TEXT_RANGECRITCHANCE_DESC"],
					get		= function() return db.options.textdisplay.rangecritchance end,
					set		= function() db.options.textdisplay.rangecritchance = not db.options.textdisplay.rangecritchance end,
					disabled = function() return not db.options.display.text end,
				},
				rangehasterating = {
					order	= 120,
					type	= "toggle",
					name	= L["Ranged Haste Rating"],
					desc	= L["TEXT_RANGEHASTERATING_DESC"],
					get		= function() return db.options.textdisplay.rangehasterating end,
					set		= function() db.options.textdisplay.rangehasterating = not db.options.textdisplay.rangehasterating end,
					disabled = function() return not db.options.display.text end,
				},
				rangehastebonus = {
					order	= 121,
					type	= "toggle",
					name	= L["Ranged Haste Bonus"],
					desc	= L["TEXT_RANGEHASTEBONUS_DESC"],
					get		= function() return db.options.textdisplay.rangehastebonus end,
					set		= function() db.options.textdisplay.rangehastebonus = not db.options.textdisplay.rangehastebonus end,
					disabled = function() return not db.options.display.text end,
				},
				header7 = {
					order	= 125,
					type	= "header",
					name	= L["Spell Group"],
				},
				holydmg = {
					order	= 126,
					type	= "toggle",
					name	= L["Holy Damage Bonus"],
					desc	= L["TEXT_HOLYDMG_DESC"],
					get		= function() return db.options.textdisplay.holydmg end,
					set		= function() db.options.textdisplay.holydmg = not db.options.textdisplay.holydmg end,
					disabled = function() return not db.options.display.text end,
				},
				firedmg = {
					order	= 127,
					type	= "toggle",
					name	= L["Fire Damage Bonus"],
					desc	= L["TEXT_FIREDMG_DESC"],
					get		= function() return db.options.textdisplay.firedmg end,
					set		= function() db.options.textdisplay.firedmg = not db.options.textdisplay.firedmg end,
					disabled = function() return not db.options.display.text end,
				},
				naturedmg = {
					order	= 128,
					type	= "toggle",
					name	= L["Nature Damage Bonus"],
					desc	= L["TEXT_NATUREDMG_DESC"],
					get		= function() return db.options.textdisplay.naturedmg end,
					set		= function() db.options.textdisplay.naturedmg = not db.options.textdisplay.naturedmg end,
					disabled = function() return not db.options.display.text end,
				},
				frostdmg = {
					order	= 129,
					type	= "toggle",
					name	= L["Frost Damage Bonus"],
					desc	= L["TEXT_FROSTDMG_DESC"],
					get		= function() return db.options.textdisplay.frostdmg end,
					set		= function() db.options.textdisplay.frostdmg = not db.options.textdisplay.frostdmg end,
					disabled = function() return not db.options.display.text end,
				},
				shadowdmg = {
					order	= 130,
					type	= "toggle",
					name	= L["Shadow Damage Bonus"],
					desc	= L["TEXT_SHADOWDMG_DESC"],
					get		= function() return db.options.textdisplay.shadowdmg end,
					set		= function() db.options.textdisplay.shadowdmg = not db.options.textdisplay.shadowdmg end,
					disabled = function() return not db.options.display.text end,
				},
				arcanedmg = {
					order	= 131,
					type	= "toggle",
					name	= L["Arcane Damage Bonus"],
					desc	= L["TEXT_ARCANEDMG_DESC"],
					get		= function() return db.options.textdisplay.arcanedmg end,
					set		= function() db.options.textdisplay.arcanedmg = not db.options.textdisplay.arcanedmg end,
					disabled = function() return not db.options.display.text end,
				},
				healing = {
					order	= 132,
					type	= "toggle",
					name	= L["Healing Bonus"],
					desc	= L["TEXT_HEALING_DESC"],
					get		= function() return db.options.textdisplay.healing end,
					set		= function() db.options.textdisplay.healing = not db.options.textdisplay.healing end,
					disabled = function() return not db.options.display.text end,
				},
				spellhitrating = {
					order	= 133,
					type	= "toggle",
					name	= L["Spell Hit Rating"],
					desc	= L["TEXT_SPELLHITRATING_DESC"],
					get		= function() return db.options.textdisplay.spellhitrating end,
					set		= function() db.options.textdisplay.spellhitrating = not db.options.textdisplay.spellhitrating end,
					disabled = function() return not db.options.display.text end,
				},
				spellhitbonus = {
					order	= 134,
					type	= "toggle",
					name	= L["Spell Hit Bonus"],
					desc	= L["TEXT_SPELLHITBONUS_DESC"],
					get		= function() return db.options.textdisplay.spellhitbonus end,
					set		= function() db.options.textdisplay.spellhitbonus = not db.options.textdisplay.spellhitbonus end,
					disabled = function() return not db.options.display.text end,
				},
				spellcritrating = {
					order	= 135,
					type	= "toggle",
					name	= L["Spell Crit Rating"],
					desc	= L["TEXT_SPELLCRITRATING_DESC"],
					get		= function() return db.options.textdisplay.spellcritrating end,
					set		= function() db.options.textdisplay.spellcritrating = not db.options.textdisplay.spellcritrating end,
					disabled = function() return not db.options.display.text end,
				},
				spellcritbonus = {
					order	= 136,
					type	= "toggle",
					name	= L["Spell Crit Bonus"],
					desc	= L["TEXT_SPELLCRITBONUS_DESC"],
					get		= function() return db.options.textdisplay.spellcritbonus end,
					set		= function() db.options.textdisplay.spellcritbonus = not db.options.textdisplay.spellcritbonus end,
					disabled = function() return not db.options.display.text end,
				},
				holycrit = {
					order	= 137,
					type	= "toggle",
					name	= L["Holy Crit Chance"],
					desc	= L["TEXT_HOLYCRIT_DESC"],
					get		= function() return db.options.textdisplay.holycrit end,
					set		= function() db.options.textdisplay.holycrit = not db.options.textdisplay.holycrit end,
					disabled = function() return not db.options.display.text end,
				},
				firecrit = {
					order	= 138,
					type	= "toggle",
					name	= L["Fire Crit Chance"],
					desc	= L["TEXT_FIRECRIT_DESC"],
					get		= function() return db.options.textdisplay.firecrit end,
					set		= function() db.options.textdisplay.firecrit = not db.options.textdisplay.firecrit end,
					disabled = function() return not db.options.display.text end,
				},
				naturecrit = {
					order	= 139,
					type	= "toggle",
					name	= L["Nature Crit Chance"],
					desc	= L["TEXT_NATURECRIT_DESC"],
					get		= function() return db.options.textdisplay.naturecrit end,
					set		= function() db.options.textdisplay.naturecrit = not db.options.textdisplay.naturecrit end,
					disabled = function() return not db.options.display.text end,
				},
				frostcrit = {
					order	= 140,
					type	= "toggle",
					name	= L["Frost Crit Chance"],
					desc	= L["TEXT_FROSTCRIT_DESC"],
					get		= function() return db.options.textdisplay.frostcrit end,
					set		= function() db.options.textdisplay.frostcrit = not db.options.textdisplay.frostcrit end,
					disabled = function() return not db.options.display.text end,
				},
				shadowcrit = {
					order	= 141,
					type	= "toggle",
					name	= L["Shadow Crit Chance"],
					desc	= L["TEXT_SHADOWCRIT_DESC"],
					get		= function() return db.options.textdisplay.shadowcrit end,
					set		= function() db.options.textdisplay.shadowcrit = not db.options.textdisplay.shadowcrit end,
					disabled = function() return not db.options.display.text end,
				},
				arcanecrit = {
					order	= 142,
					type	= "toggle",
					name	= L["Arcane Crit Chance"],
					desc	= L["TEXT_ARCANECRIT_DESC"],
					get		= function() return db.options.textdisplay.arcanecrit end,
					set		= function() db.options.textdisplay.arcanecrit = not db.options.textdisplay.arcanecrit end,
					disabled = function() return not db.options.display.text end,
				},
				spellhasterating = {
					order	= 143,
					type	= "toggle",
					name	= L["Spell Haste Rating"],
					desc	= L["TEXT_SPELLHASTERATING_DESC"],
					get		= function() return db.options.textdisplay.spellhasterating end,
					set		= function() db.options.textdisplay.spellhasterating = not db.options.textdisplay.spellhasterating end,
					disabled = function() return not db.options.display.text end,
				},
				spellhastebonus = {
					order	= 144,
					type	= "toggle",
					name	= L["Spell Haste Bonus"],
					desc	= L["TEXT_SPELLHASTEBONUS_DESC"],
					get		= function() return db.options.textdisplay.spellhastebonus end,
					set		= function() db.options.textdisplay.spellhastebonus = not db.options.textdisplay.spellhastebonus end,
					disabled = function() return not db.options.display.text end,
				},
				spellpenetration = {
					order	= 145,
					type	= "toggle",
					name	= L["Spell Penetration"],
					desc	= L["TEXT_SPELLPEN_DESC"],
					get		= function() return db.options.textdisplay.spellpenetration end,
					set		= function() db.options.textdisplay.spellpenetration = not db.options.textdisplay.spellpenetration end,
					disabled = function() return not db.options.display.text end,
				},
				manaregen = {
					order	= 146,
					type	= "toggle",
					name	= L["Mana Regen"],
					desc	= L["TEXT_MANAREGEN_DESC"],
					get		= function() return db.options.textdisplay.manaregen end,
					set		= function() db.options.textdisplay.manaregen = not db.options.textdisplay.manaregen end,
					disabled = function() return not db.options.display.text end,
				},
				castingregen = {
					order	= 147,
					type	= "toggle",
					name	= L["Casting Mana Regen"],
					desc	= L["TEXT_CASTINGREGEN_DESC"],
					get		= function() return db.options.textdisplay.castingregen end,
					set		= function() db.options.textdisplay.castingregen = not db.options.textdisplay.castingregen end,
					disabled = function() return not db.options.display.text end,
				},
			},
		}
	end
	return textOptions
end

local options = nil
local function fullOptions()
	if (not options) then
		options = {
			type = "group",
			name = MODNAME,
			args = {
				general = {
					order	= 1,
					type	= "group",
					name	= L["General Information"],
					desc	= L["GENERAL_INFO_DESC"],
					args	= {
						header1 = {
							order	= 11,
							type	= "header",
							name	= L["General Information"],
						},
						desc = {
							order	= 12,
							type	= "description",
							name	= L["GENERAL_INFO_DESC"],
						},
						author = {
							order	= 13,
							type	= "description",
							name	= L["Author : "] .. addonauthor .. "\n",
						},
						version = {
							order	= 14,
							type	= "description",
							name	= L["Version : "] .. addonversion .. "\n",
						},
						builddate = {
							order	= 15,
							type	= "description",
							name	= L["Build Date : "] .. builddate .. "\n",
						},
						dbversion = {
							order	= 15,
							type	= "description",
							name	= L["Database Version : "] .. db.options.display.dbVersion .. "\n",
						},
						translators = {
							order	= 16,
							type	= "description",
							name	= string.format( L["Helpful Translators (thank you) : %s"], " \n" )
						},
					},
				},
			},
		}
	end
	
	for k,v in pairs(modularOptions) do
		options.args[k] = (type(v) == "function") and v() or v
	end

	return options
end

function addon:SetupOptions()
	AceConfigReg:RegisterOptionsTable(MODNAME, fullOptions)
	self.optionsFrame = AceConfigDialog:AddToBlizOptions(MODNAME, nil, nil, "general")

	-- Fill up our modular options...
	self:RegisterModuleOptions("Display", giveDisplay(), L["Display Options"])
	self:RegisterModuleOptions("Tooltip", giveTooltip(), L["Tooltip Options"])
	self:RegisterModuleOptions("Text", giveText(), L["Text Options"])
end

function addon:RegisterModuleOptions(name, optionsTable, displayName)
	modularOptions[name] = optionsTable
	self.optionsFrame[name] = AceConfigDialog:AddToBlizOptions(MODNAME, displayName, MODNAME, name)
end

function addon:ShowConfig()
	-- Open the profiles tab before, so the menu expands
	InterfaceOptionsFrame_OpenToCategory( self.optionsFrame["Display"] )
end

function addon.updateCachedStats()
-- Strength, Agility, Stamina, Intellect, Spirit
	_, cachedValues[ L["Strength"] ]			= UnitStat( "player", 1 )
	_, cachedValues[ L["Agility"] ]				= UnitStat( "player", 2 )
	_, cachedValues[ L["Stamina"] ]				= UnitStat( "player", 3 )
	_, cachedValues[ L["Intellect"] ]			= UnitStat( "player", 4 )
	_, cachedValues[ L["Spirit"] ]				= UnitStat( "player", 5 )
-- max Health and max Power (mana, rage, power or runic power)
	cachedValues[ L["Health"] ]					= UnitHealthMax( "player" )
	cachedValues[ L["Power"] ]					= UnitPowerMax( "player" )
-- Armor, Defense, Parry, Dodge, Block, Avoidance
	_, cachedValues[ L["Armor"] ]				= UnitArmor( "player" )
	cachedValues[ L["Defense" ] ]				= GetCombatRating( 2 )
	cachedValues[ L["Defense Bonus" ] ]			= GetCombatRatingBonus( 2 )
	local baseDefense, armorDefense				= UnitDefense( "player" )
	cachedValues[ L["Defense Rating" ] ]		= baseDefense + armorDefense
	cachedValues[ L["Parry Rating"] ]			= GetCombatRating( 4 )
	cachedValues[ L["Parry Bonus"] ]			= GetCombatRatingBonus( 4 )
	cachedValues[ L["Parry Chance"] ]			= GetParryChance()
	cachedValues[ L["Dodge Rating"] ]			= GetCombatRating( 3 )
	cachedValues[ L["Dodge Bonus"] ]			= GetCombatRatingBonus( 3 )
	cachedValues[ L["Dodge Chance"] ]			= GetDodgeChance()
	cachedValues[ L["Block Rating"] ]			= GetCombatRating( 5 )
	cachedValues[ L["Block Bonus"] ]			= GetCombatRatingBonus( 5 )
	cachedValues[ L["Block Percentage"] ]		= GetBlockChance()
	cachedValues[ L["Avoidance"] ]				= GetDodgeChance() + GetBlockChance() + GetParryChance()
-- Resistances
--	_, cachedValues[ L["Physical Resistance"] ]	= UnitResistance( "player", 0 )
	_, cachedValues[ L["Holy Resistance"] ]		= UnitResistance( "player", 1 )
	_, cachedValues[ L["Fire Resistance"] ]		= UnitResistance( "player", 2 )
	_, cachedValues[ L["Nature Resistance"] ]	= UnitResistance( "player", 3 )
	_, cachedValues[ L["Frost Resistance"] ]	= UnitResistance( "player", 4 )
	_, cachedValues[ L["Shadow Resistance"] ]	= UnitResistance( "player", 5 )
	_, cachedValues[ L["Arcane Resistance"] ]	= UnitResistance( "player", 6 )
-- Melee Group ( Dmg, Speed, Power, Hit, Crit, Haste, Armor Penetration )
	local _, minDmg, maxDmg, minOffDmg, maxOffDmg	= UnitDamage( "player" )
	cachedValues[ L["Main Hand Damage"] ]		= minDmg .. "-" .. maxDmg
	cachedValues[ L["Off Hand Damage"] ]		= minOffDmg .. "-" .. maxOffDmg
	local speed, offSpeed						= UnitAttackSpeed( "player" )
	cachedValues[ L["Main Hand Speed"] ]		= speed
	cachedValues[ L["Off Hand Speed"] ]			= offSpeed
	local baseAP, plusAP, minusAP				= UnitAttackPower( "player" )
	cachedValues[ L["Attack Power"] ]			= baseAP + plusAP + minusAP
	cachedValues[ L["Hit Rating"] ]				= GetCombatRating( 6 )
	cachedValues[ L["Hit Bonus"] ]				= GetCombatRatingBonus( 6 )
	cachedValues[ L["Crit Rating"] ]			= GetCombatRating( 9 )
	cachedValues[ L["Crit Bonus"] ]				= GetCombatRatingBonus( 9 )
	cachedValues[ L["Crit Chance"] ]			= GetCritChance()
	cachedValues[ L["Haste Rating"] ]			= GetCombatRating( 18 )
	cachedValues[ L["Haste Bonus"] ]			= GetCombatRatingBonus( 18 )
	cachedValues[ L["Armor Penetration"] ]		= GetArmorPenetration()
	cachedValues[ L["Expertise Rating"] ]		= GetExpertise()
	cachedValues[ L["Expertise Bonus"] ]		= GetExpertisePercent()
	cachedValues[ L["Be Parried Chance"] ]	= GetCombatRatingBonus( 24 )
-- Ranged Group (Dmg, Speed, Power, Hit, Crit, Haste)
	local speed, lowDmg, maxDmg = UnitRangedDamage( "player" )
	cachedValues[ L["Ranged Speed"] ]			= speed
	cachedValues[ L["Ranged Damage"] ]			= lowDmg .. "-" .. maxDmg
	local baseRAP, plusRAP, minusRAP			= UnitRangedAttackPower( "player" )
	cachedValues[ L["Ranged Attack Power"] ]	= baseRAP + plusRAP + minusRAP
	cachedValues[ L["Ranged Hit Rating"] ]		= GetCombatRating( 7 )
	cachedValues[ L["Ranged Hit Bonus"] ]		= GetCombatRatingBonus( 7 )
	cachedValues[ L["Ranged Crit Rating"] ]		= GetCombatRating( 10 )
	cachedValues[ L["Ranged Crit Bonus"] ]		= GetCombatRatingBonus( 10 )
	cachedValues[ L["Ranged Crit Chance"] ]		= GetRangedCritChance()
	cachedValues[ L["Ranged Haste Rating"] ]	= GetCombatRating( 19 )
	cachedValues[ L["Ranged Haste Bonus"] ]		= GetCombatRating( 19 )
-- Spell Group ( spellbonus{}, healbonus, spellhit, spellcrit{}, manaregen, manaregenfsr, spellpenetration )
	cachedValues[ L["Holy Damage Bonus"] ]		= GetSpellBonusDamage( 2 )
	cachedValues[ L["Fire Damage Bonus"] ]		= GetSpellBonusDamage( 3 )
	cachedValues[ L["Nature Damage Bonus"] ]	= GetSpellBonusDamage( 4 )
	cachedValues[ L["Frost Damage Bonus"] ]		= GetSpellBonusDamage( 5 )
	cachedValues[ L["Shadow Damage Bonus"] ]	= GetSpellBonusDamage( 6 )
	cachedValues[ L["Arcane Damage Bonus"] ]	= GetSpellBonusDamage( 7 )
	cachedValues[ L["Healing Bonus"] ]			= GetSpellBonusHealing()
	cachedValues[ L["Spell Hit Rating"] ]		= GetCombatRating( 8 )
	cachedValues[ L["Spell Hit Bonus"] ]		= GetCombatRatingBonus( 8 )
	cachedValues[ L["Spell Crit Rating"] ]		= GetCombatRating( 11 )
	cachedValues[ L["Spell Crit Bonus"] ]		= GetCombatRatingBonus( 11 )
	cachedValues[ L["Holy Crit Chance"] ]		= GetSpellCritChance( 2 )
	cachedValues[ L["Fire Crit Chance"] ]		= GetSpellCritChance( 3 )
	cachedValues[ L["Nature Crit Chance"] ]		= GetSpellCritChance( 4 )
	cachedValues[ L["Frost Crit Chance"] ]		= GetSpellCritChance( 5 )
	cachedValues[ L["Shadow Crit Chance"] ]		= GetSpellCritChance( 6 )
	cachedValues[ L["Arcane Crit Chance"] ]		= GetSpellCritChance( 7 )
	cachedValues[ L["Spell Haste Rating"] ]		= GetCombatRating( 20 )
	cachedValues[ L["Spell Haste Bonus"] ]		= GetCombatRatingBonus( 20 )
	cachedValues[ L["Spell Penetration"] ]		= GetSpellPenetration()
	local manaregen, castingregen				= GetManaRegen()
	cachedValues[ L["Mana Regen"] ]				= manaregen
	cachedValues[ L["Casting Mana Regen"] ]		= castingregen
end

function addon.startScanning()
	local totalElapsed = 0
	local refreshElapsed = 0

	if ( not AllStats_ScannerFrame ) then
		AllStats_ScannerFrame = CreateFrame( "Frame" )
	end

	AllStats_ScannerFrame:SetScript( "OnUpdate", function(self, elapsed)
		totalElapsed = totalElapsed + elapsed
		refreshElapsed = refreshElapsed + elapsed
		-- update once every 30 seconds. I may change this later
		if (totalElapsed < 2 ) then
			-- update our current cached values
			addon.updateCachedStats()
		elseif ( totalElapsed > 30 ) then
			totalElapsed = 0
		end
	end )
end


function addon:OnInitialize()
	-- Do Frame Creation stuff here
	-- addon:CreateFrames()
end

function addon:OnEnable()
    -- Called when the addon is loaded
	addon.me = UnitName( "player" )

	self.db = LibStub( "AceDB-3.0" ):New( MODNAME .. "_DB", nil, "Default" )
	-- do we have any data yet; for this toon? If not, init its table
	if not ( self.db.factionrealm[addon.me] ) then
		self.db.factionrealm[addon.me] = {
		-- Your saved variables stuff here
			options = {
				display = {
					debug = false,
					tipscale = 1,
					text = true,
					dbVersion = "1.1",
				},
				tooltipgroups = {
					basic = true,
					resist = true,
					defense = true,
					melee = true,
					ranged = true,
					spell = true,
				},
				textdisplay = {					
					shuffle = true,
					interval = 10,
					-- Basic Group
					strength = false,
					agility = false,
					stamina = false,
					intellect = false,
					spirit = false,
					health = false,
					power = false,
					-- Defense Group
					armor = false,
					defense = false,
					defensebonus = false,
					defenserating = false,
					parryrating = false,
					parrybonus = false,
					parrychance = false,
					dodgerating = false,
					dodgebonus = false,
					dodgechance = false,
					blockrating = false,
					blockbonus = false,
					blockpercent = false,
					avoidance = false,
					-- Resistance Group
					holyresist = false,
					fireresist = false,
					natureresist = false,
					frostresist = false,
					shadowresist = false,
					arcaneresist = false,
					-- Melee Group
					maindmg = false,
					offdmg = false,
					mainspeed = false,
					offspeed = false,
					attpower = false,
					hitrating = false,
					hitbonus = false,
					critrating = false,
					critbonus = false,
					critchance = false,
					hasterating = false,
					hastebonus = false,
					armorpen = false,
					expertrating = false,
					expertbonus = false,
					beparrychance = false,
					-- Ranged Group
					rangespeed = false,
					rangedmg = false,
					rangeAP = false,
					rangehitrating = false,
					rangehitbonus = false,
					rangecritrating = false,
					rangecritbonus = false,
					rangecritchance = false,
					rangehasterating = false,
					rangehastebonus = false,
					-- Spell Group
					holydmg = false,
					firedmg = false,
					naturedmg = false,
					frostdmg = false,
					shadowdmg = false,
					arcanedmg = false,
					healing = false,
					spellhitrating = false,
					spellhitbonus = false,
					spellcritrating = false,
					spellcritbonus = false,
					holycrit = false,
					firecrit = false,
					naturecrit = false,
					frostcrit = false,
					shadowcrit = false,
					arcanecrit = false,
					spellhasterating = false,
					spellhastebonus = false,
					spellpenetration = false,
					manaregen = false,
					castingregen = false,
				},
			},
		}
	else
		-- check if we need to upgrade the db
		addon.upgradeDB()
	end
	db = self.db.factionrealm[addon.me]

	-- tooltip setup and scaling
	tooltip = LTT:Acquire( MODNAME.."TT", 3, "LEFT", "CENTER", "RIGHT" ) -- or GameTooltip
	tooltip:SetScale( db.options.display.tipscale )

	addon:SetupOptions()
	-- Event listeners
    -- addon:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

	-- kick off our timed scanning of stats
	addon.startScanning()
end

--[[
************************************************************************
CHANGELOG:

11/20/08 : 
	Initial version
************************************************************************
]]--